Module: CPEE

Defined in:
lib/cpee/transformation/target.rb,
lib/cpee/transformation/cpee.rb,
lib/cpee/transformation/ptml.rb,
lib/cpee/transformation/bpmn2.rb,
lib/cpee/transformation/mermaid.rb,
lib/cpee/transformation/text-bf.rb,
lib/cpee/transformation/dataflow.rb,
lib/cpee/transformation/graphviz.rb,
lib/cpee/transformation/beautiful.rb,
lib/cpee/transformation/structures.rb,
lib/cpee/transformation/transformer.rb,
lib/cpee/transformation/implementation.rb,
lib/cpee/transformation/text-df-PO-reduced.rb,
lib/cpee/transformation/text-df-PO-extended.rb

Overview

This file is part of CPEE-TRANSFORMATION.

CPEE-TRANSFORMATION is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

CPEE-TRANSFORMATION is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with CPEE-TRANSFORMATION (file COPYING in the main directory). If not, see http://www.gnu.org/licenses/.

Defined Under Namespace

Modules: Transformation